# KnifeGrenade by David Bowland
# ./addons/eventscripts/knifegrenade/knifegrenade.py

# >>> To configure this addon please see knifegrenade.cfg <<<

"""
Allows players to pull the pin on the grenade of an opponent by attacking with a knife.
Requires only ES 2.0+
"""


import cfglib
import es
import gamethread
import langlib
import playerlib
import random
import usermsg

from path import path

import psyco
psyco.full()


info = es.AddonInfo()
info.name     = 'KnifeGrenade'
info.version  = '6'
info.basename = 'knifegrenade'
info.url      = 'http://addons.eventscripts.com/addons/view/' + info.basename
info.author   = 'SuperDave'


addonpath = path(es.getAddonPath(info.basename))

###

config = cfglib.AddonCFG(addonpath.joinpath(info.basename + '.cfg'))

config.text(info.name + ' release %s options' % info.version)
config.text('./addons/eventscripts/%(basename)s/%(basename)s.cfg' % {'basename': info.basename})
config.text(info.url)
config.text('')
config.text('Load this addon with: es_load ' + info.basename)
config.text('\n')
config.text('To configure language settings for %s please see %s_languages.ini' % (info.name, info.basename))
config.text('\n')
config.text('This addon will turn on eventscripts_noisy. More information on eventscripts_noisy can be found here:')
config.text('http://www.eventscripts.com/pages/Eventscripts_noisy')
config.text('')
config.text('Thanks to 7355608 for this idea! PinPull can be found here:')
config.text('http://addons.eventscripts.com/addons/view/12412')
config.text('\n')

config.text('***** Pin pull options *****')
config.text('If a player is attacked with a knife and has a HE grenade, that grenade will explode in a few seconds.')
config.text('Credit for the explosion will go to the player who attacked with the knife as env_explosion damage.')
cvar_pinpull_givegrenade    = config.cvar('knifegrenade_pinpull_givegrenade',  1, "0 = no change, 1 = if the attacker has a HE grenade and the victim doesn't, the attacker's grenade will be given to the victim, 2 = always give the victim a grenade")
cvar_pinpull_explodeondeath = config.cvar('knifegrenade_pinpull_explodeondeath', 1, '0 = no change, 1 = players with a ticking grenade that die will explode when they die')
config.text('\n')

config.text('***** Grenade detonation options *****')
cvar_detonate_delay     = config.cvar('knifegrenade_detonate_delay',     4,   'Number of seconds after a pin pull a grenade explodes')
cvar_detonate_radius    = config.cvar('knifegrenade_detonate_radius',    350, 'Radius of explosion when a grenade with its pin pulled explodes--default HE grenade radius is 350')
cvar_detonate_magnitude = config.cvar('knifegrenade_detonate_magnitude', 300, 'Radius of explosion when a grenade with its pin pulled explodes--default HE grenade magnitude is 100')
config.text('\n')

config.text('***** Sound options *****')
cvar_sound_percentpinpull  = config.cvar('knifegrenade_sound_percentpinpull',  85, 'Percent of the map that can hear grenade pin pulls--set to 0 to eliminate')
cvar_sound_percenttick     = config.cvar('knifegrenade_sound_percenttick',     60, 'Percent of the map that can hear grenade ticks--set to 0 to eliminate')
cvar_sound_percentdetonate = config.cvar('knifegrenade_sound_percentdetonate', 45, 'Percent of the map that can hear grenade explosions--set to 0 to eliminate')
config.text('\n')

config.text('***** Announcement options *****')
cvar_announce_victim   = config.cvar('knifegrenade_announce_victim',   0, '0 = no change, 1 = announce to player when the pin is pulled on his or her grenade')
cvar_announce_attacker = config.cvar('knifegrenade_announce_attacker', 1, '0 = no change, 1 = announce to player when he or she pulls the pin on another player\'s grenade')
cvar_announce_all      = config.cvar('knifegrenade_announce_all',      2, '0 = no change, 1 = announce to all players when a player has a grenade pin pulled in chat area, 2 = announce to all players when a player has a grenade pin pulled on the console')

config.write()


lastgive = es.ServerVar('eventscripts_lastgive')

###

ini = cfglib.AddonINI(addonpath.joinpath(info.basename + '_languages.ini'))

ini.setInitialComments([
 '# ./addons/eventscripts/%(basename)s/%(basename)s_languages.ini' % {'basename': info.basename},
 '# >>> For general addon configuration see ' + info.basename + '.cfg <<<',
 '',
 '# DO NOT translate words with $ or # in front!',
 '# For a list of language abbreviations please see ./addons/eventscripts/_libs/python/deflangs.ini'])

ini.addGroup('pin pull')
ini.addValueToGroup('pin pull', 'en', "#defaultYou pulled the pin on #lightgreen$victim#default's grenade!")
ini.addValueToGroup('pin pull', 'de', "#defaultDu hast den Ring von #lightgreen$victim#default's Granate entfernt!")

ini.addGroup('pin pulled')
ini.addValueToGroup('pin pulled', 'en', '#lightgreen$attacker#default pulled the pin on your grenade!')
ini.addValueToGroup('pin pulled', 'de', '#lightgreen$attacker#default hat den Ring von Deiner Granate entfernt!')

ini.addGroup('alert all players')
ini.addValueToGroup('alert all players', 'en',
 "#lightgreen$attacker#default pulled the pin on #lightgreen$victim#default's grenade!")
ini.addValueToGroup('alert all players', 'de',
 "#lightgreen$attacker#default hat den Ring von #lightgreen$victim#default's Granate entfernt!")

ini.write()


lang_text = langlib.Strings(ini)

###

class TickingGrenadeManager(object):
   count = 0

   def __init__(self, userid, index, attacker):
      self.userid    = userid
      self.handle    = es.getplayerhandle(userid)
      self.index     = index
      self.attacker  = attacker
      self.delayname = 'knifegrenade_' + str(userid)

      # Play initial sound
      self.playSound('weapons/pinpull.wav', cvar_sound_percentpinpull)

      # Start a delay to handle the first grenade "tick"
      gamethread.delayedname(1, self.delayname, self.tick)

   def removeDelay(self):
      """ Cancels the delay associated with the player """
      gamethread.cancelDelayed(self.delayname)

   def tick(self):
      """ Increases the player's grenade "timer" a single tick """
      if (self.index in es.createentitylist('weapon_hegrenade') and
       es.getindexprop(self.index, 'CBaseEntity.m_hOwnerEntity') == self.handle):
         # Tick count + 1
         self.count += 1

         # Should we detonate?
         if self.count >= cvar_detonate_delay:
            self.explode()
            kg.removePlayer(self.userid)

         # No detonation, so play a "tick" sound and start the fun again
         else:
            self.playSound('weapons/grenade/tick1.wav', cvar_sound_percenttick)
            gamethread.delayedname(1, self.delayname, self.tick)

      # The grenade we wanted is gone. No telling what happened to it, but I suppose we don't care.
      # In any case, we need to remove the player from our dictionary
      else:
         kg.removePlayer(self.userid)

   def explode(self):
      """ "Detonates" the player """
      # If the player still has his or her grenade, we need to remove it as it has "exploded"
      if self.index in es.createentitylist('weapon_hegrenade'):
         es.remove(self.index)

      # Create the explosion entity and set its values
      index = es.createentity('env_explosion') # Create the explosion
      es.entitysetvalue(index, 'origin',
       es.getplayerprop(self.userid, 'CBaseEntity.m_vecOrigin').replace(',', ' ')) # Location
      es.entitysetvalue(index, 'imagnitude', cvar_detonate_magnitude) # Magnitude
      es.entitysetvalue(index, 'iradiusoverride', cvar_detonate_radius) # Radius
      es.entitysetvalue(index, 'targetname', 'knifegrenade_explosion') # Unique name
      es.entitysetvalue(index, 'classname', 'weapon_hegrenade') # Make this explosion credit weapon_hegrenade
      es.spawnentity(index)
      es.setindexprop(index, 'CBaseEntity.m_hOwnerEntity', es.getplayerhandle(self.attacker)) # Owner (attacker)

      # Boom!
      es.fire(self.userid, 'knifegrenade_explosion', 'explode')
      # We're done with the explosion, so we can remove it
      es.fire(self.userid, 'knifegrenade_explosion', 'kill')

      # Can't forget to play a sound!
      self.playSound('weapons/hegrenade/explode%s.wav' % random.randint(3, 5), cvar_sound_percentdetonate)

   def playSound(self, sound, sound_percent):
      """ Emits a sound from the player at a given volume """
      if sound_percent:
         es.emitsound('player', self.userid, sound, 1.0, (100. - sound_percent) / 100)

###

class KnifeGrenadeManager(object):
   def __init__(self):
      self.players = {}

   def knifeAttack(self, userid, attacker):
      """ Creates a "timer" delay for the victim's HE grenade to explode """
      # Does this player already have a delay?
      if userid in self.players:
         return

      # We need to know if the player has a grenade
      hegrenades = es.createentitylist('weapon_hegrenade')
      index = self.findGrenade(userid, hegrenades)

      # If the player has no grenade, we might be able to give one to him or her
      if not index:
         if cvar_pinpull_givegrenade == 2:
            es.give(userid, 'weapon_hegrenade')
            index = int(lastgive)

         elif cvar_pinpull_givegrenade:
            index = self.findGrenade(attacker, hegrenades)

            if index:
               es.remove(index)

               es.give(userid, 'weapon_hegrenade')
               index = int(lastgive)

      # No grenade at this point means no delay
      if not index:
         return

      # Start the magic!
      self.players[userid] = TickingGrenadeManager(userid, index, attacker)

      # Announce the pin-pull action
      tokens = {'attacker': es.getplayername(attacker), 'victim': es.getplayername(userid)}

      if cvar_announce_victim:
         es.tell(userid, '#multi', lang_text('pin pulled', tokens, playerlib.getPlayer(userid).get('lang')))

      if cvar_announce_attacker:
         es.tell(attacker, '#multi', lang_text('pin pull', tokens, playerlib.getPlayer(attacker).get('lang')))

      if cvar_announce_all == 2:
         for loop_userid in es.getUseridList():
            usermsg.echo(loop_userid, self._removeTags(lang_text('alert all players', tokens, playerlib.getPlayer(loop_userid).get('lang'))))

      elif cvar_announce_all:
         for loop_userid in es.getUseridList():
            es.tell(loop_userid, '#multi', lang_text('alert all players', tokens, playerlib.getPlayer(loop_userid).get('lang')))

   def playerDeath(self, userid, weapon):
      """ Removes the player from our list of delays, exploding the player if necessary """
      # If this isn't a player with a delay then we don't care
      if userid not in self.players:
         return

      # If we need to explode and weren't killed by a knife, let's explode
      if cvar_pinpull_explodeondeath and weapon != 'knife':
         self.players[userid].explode()

      # Remove the player's delay
      self.removePlayer(userid)

   def playerDisconnect(self, userid):
      """ Removes all references to the disconnecting player """
      self.removePlayer(userid)
      self.removeAttacker(userid)

   def clearDelays(self):
      """ Clears all outstanding delays """
      for userid in self.players:
         self.players[userid].removeDelay()

      self.players.clear()

   def removePlayer(self, userid):
      """ Removes the player from the dictionary, including any delays """
      if userid in self.players:
         self.players[userid].removeDelay()
         del self.players[userid]

   def removeAttacker(self, attacker):
      """ Removes the delays caused by the given attacker """
      for userid in filter(lambda x: self.players[x].attacker == attacker, self.players):
         self.players[userid].removeDelay()
         del self.players[userid]

   @staticmethod
   def findGrenade(userid, hegrenades):
      """ Returns the index of the player's HE grenade or zero if the player has no grenade """
      handle = es.getplayerhandle(userid)
      for index in hegrenades:
         if es.getindexprop(index, 'CBaseEntity.m_hOwnerEntity') == handle:
            return index

      # No grenade found
      return 0

   @staticmethod
   def _removeTags(text):
      """ Removes #lightgreen, #darkgreen, #green and #default tags from the supplied string """
      return reduce(lambda t, r: t.replace(r, ''), ('#lightgreen', '#darkgreen', '#green', '#default'), text)

kg = KnifeGrenadeManager()

###

def load():
   config.execute()

   es.doblock('corelib/noisy_on')


def es_map_start(event_var):
   kg.clearDelays()


def round_start(event_var):
   kg.clearDelays()


def player_hurt(event_var):
   if event_var['weapon'] == 'knife' and event_var['es_userteam'] != event_var['es_attackerteam']:
      kg.knifeAttack(int(event_var['userid']), int(event_var['attacker']))


def player_death(event_var):
   kg.playerDeath(int(event_var['userid']), event_var['weapon'])


def player_disconnect(event_var):
   kg.playerDisconnect(int(event_var['userid']))


def weapon_fire(event_var):
   if event_var['weapon'] == 'hegrenade':
      kg.removePlayer(int(event_var['userid']))


def hegrenade_detonate(event_var):
   kg.removePlayer(int(event_var['userid']))


def unload():
   kg.clearDelays()

   es.doblock('corelib/noisy_off')

   for cvar in config.getCvars():
      es.ServerVar(cvar).set(0)